# -*- coding: utf-8 -*-
"""
Matplotlib Backend Implementation.

Provides the Matplotlib implementation of the PlotBackend interface.
"""

from __future__ import annotations

from typing import Any, Dict, List, Optional, Tuple, Union

import pandas as pd
import numpy as np

from biblium.plotting.backends.base import PlotBackend
from biblium.plotting.config import PlotConfig


class MatplotlibBackend(PlotBackend):
    """
    Matplotlib implementation of PlotBackend.
    
    This backend uses matplotlib for all plotting operations.
    """
    
    name = "matplotlib"
    
    def __init__(self, config: Optional[PlotConfig] = None):
        super().__init__(config)
        
        # Import matplotlib
        import matplotlib.pyplot as plt
        import matplotlib
        
        self.plt = plt
        self.matplotlib = matplotlib
    
    def _create_figure(self, config: PlotConfig) -> Tuple[Any, Any]:
        """Create figure and axes with config settings."""
        # Convert pixels to inches (assuming 100 dpi for screen)
        width_inches = config.width / config.dpi
        height_inches = config.height / config.dpi
        
        fig, ax = self.plt.subplots(
            figsize=(width_inches, height_inches),
            facecolor=config.background_color,
        )
        
        ax.set_facecolor(config.plot_background_color)
        
        return fig, ax
    
    def _apply_config(self, ax: Any, config: PlotConfig) -> None:
        """Apply config settings to axes."""
        # Title
        if config.title:
            ax.set_title(
                config.title,
                fontsize=config.title_font_size,
                fontweight=config.title_font_weight,
                color=config.title_color,
                loc=config.title_position,
            )
        
        # Axis labels
        if config.xlabel:
            ax.set_xlabel(
                config.xlabel,
                fontsize=config.xlabel_font_size,
                color=config.label_color,
            )
        
        if config.ylabel:
            ax.set_ylabel(
                config.ylabel,
                fontsize=config.ylabel_font_size,
                color=config.label_color,
            )
        
        # Axis limits
        if config.xlim:
            ax.set_xlim(config.xlim)
        if config.ylim:
            ax.set_ylim(config.ylim)
        
        # Axis scale
        if config.xscale != "linear":
            ax.set_xscale(config.xscale)
        if config.yscale != "linear":
            ax.set_yscale(config.yscale)
        
        # Invert axes
        if config.invert_xaxis:
            ax.invert_xaxis()
        if config.invert_yaxis:
            ax.invert_yaxis()
        
        # Tick settings
        ax.tick_params(
            axis="x",
            labelsize=config.xtick_font_size,
            labelcolor=config.tick_color,
            rotation=config.xtick_rotation,
        )
        ax.tick_params(
            axis="y",
            labelsize=config.ytick_font_size,
            labelcolor=config.tick_color,
            rotation=config.ytick_rotation,
        )
        
        if not config.show_xticks:
            ax.set_xticks([])
        if not config.show_yticks:
            ax.set_yticks([])
        
        # Grid
        if config.show_grid:
            linestyle = {"solid": "-", "dashed": "--", "dotted": ":"}
            ax.grid(
                True,
                color=config.grid_color,
                alpha=config.grid_alpha,
                linestyle=linestyle.get(config.grid_style, "--"),
                linewidth=config.grid_width,
                axis=config.grid_axis,
            )
        else:
            ax.grid(False)
        
        # Spines
        ax.spines["top"].set_visible(config.show_top_spine)
        ax.spines["right"].set_visible(config.show_right_spine)
        ax.spines["bottom"].set_visible(config.show_bottom_spine)
        ax.spines["left"].set_visible(config.show_left_spine)
        
        for spine in ax.spines.values():
            spine.set_color(config.spine_color)
            spine.set_linewidth(config.spine_width)
    
    def _add_legend(self, ax: Any, config: PlotConfig) -> None:
        """Add legend with config settings."""
        if not config.show_legend:
            return
        
        # Map position names
        loc_map = {
            "outside right": "center left",
            "outside top": "lower center",
        }
        loc = loc_map.get(config.legend_position, config.legend_position)
        
        bbox = None
        if config.legend_position == "outside right":
            bbox = (1.02, 0.5)
        elif config.legend_position == "outside top":
            bbox = (0.5, 1.02)
        
        legend = ax.legend(
            loc=loc,
            fontsize=config.legend_font_size,
            title=config.legend_title,
            frameon=config.legend_frameon,
            ncol=config.legend_ncol,
            bbox_to_anchor=bbox,
        )
        
        if config.legend_title and legend:
            legend.get_title().set_fontsize(config.legend_title_font_size)
    
    def bar(
        self,
        data: pd.DataFrame,
        x: str,
        y: str,
        config: Optional[PlotConfig] = None,
        **kwargs,
    ) -> Any:
        """Create a vertical bar chart."""
        config = self._merge_config(config)
        fig, ax = self._create_figure(config)
        
        # Use single color for all bars
        color = self._get_colors(1, config)[0]
        
        bars = ax.bar(
            data[x],
            data[y],
            width=config.bar_width,
            color=color,
            alpha=config.alpha,
            edgecolor=config.edge_color,
            linewidth=config.edge_width,
        )
        
        # Add value labels
        if config.show_values:
            for bar in bars:
                height = bar.get_height()
                if config.value_position == "outside":
                    va = "bottom"
                    y_pos = height
                elif config.value_position == "inside":
                    va = "top"
                    y_pos = height * 0.95
                else:  # center
                    va = "center"
                    y_pos = height / 2
                
                ax.annotate(
                    config.value_format.format(height),
                    xy=(bar.get_x() + bar.get_width() / 2, y_pos),
                    ha="center",
                    va=va,
                    fontsize=config.value_font_size,
                    color=config.label_color,
                )
        
        # Apply configuration
        self._apply_config(ax, config)
        
        if config.tight_layout:
            self.plt.tight_layout()
        
        return fig
    
    def barh(
        self,
        data: pd.DataFrame,
        x: str,
        y: str,
        config: Optional[PlotConfig] = None,
        **kwargs,
    ) -> Any:
        """Create a horizontal bar chart with top items at the top."""
        config = self._merge_config(config)
        fig, ax = self._create_figure(config)
        
        # Reverse the data so top items appear at the top
        data = data.iloc[::-1].reset_index(drop=True)
        
        # Use single color for all bars
        color = self._get_colors(1, config)[0]
        
        bars = ax.barh(
            data[y],
            data[x],
            height=config.bar_width,
            color=color,
            alpha=config.alpha,
            edgecolor=config.edge_color,
            linewidth=config.edge_width,
        )
        
        # Add value labels
        if config.show_values:
            for bar in bars:
                width = bar.get_width()
                if config.value_position == "outside":
                    ha = "left"
                    x_pos = width * 1.01
                elif config.value_position == "inside":
                    ha = "right"
                    x_pos = width * 0.95
                else:  # center
                    ha = "center"
                    x_pos = width / 2
                
                ax.annotate(
                    config.value_format.format(width),
                    xy=(x_pos, bar.get_y() + bar.get_height() / 2),
                    ha=ha,
                    va="center",
                    fontsize=config.value_font_size,
                    color=config.label_color,
                )
        
        self._apply_config(ax, config)
        
        if config.tight_layout:
            self.plt.tight_layout()
        
        return fig
    
    def line(
        self,
        data: pd.DataFrame,
        x: str,
        y: Union[str, List[str]],
        config: Optional[PlotConfig] = None,
        **kwargs,
    ) -> Any:
        """Create a line chart."""
        config = self._merge_config(config)
        fig, ax = self._create_figure(config)
        
        # Handle single or multiple y columns
        y_cols = [y] if isinstance(y, str) else y
        colors = self._get_colors(len(y_cols), config)
        
        linestyle_map = {
            "solid": "-",
            "dashed": "--",
            "dotted": ":",
            "dashdot": "-.",
        }
        linestyle = linestyle_map.get(config.line_style, "-")
        
        for i, col in enumerate(y_cols):
            line_kwargs = {
                "color": colors[i],
                "linewidth": config.line_width,
                "linestyle": linestyle,
                "alpha": config.alpha,
                "label": col,
            }
            
            if config.show_markers:
                line_kwargs["marker"] = config.marker_style
                line_kwargs["markersize"] = config.marker_size
            
            ax.plot(data[x], data[col], **line_kwargs)
            
            if config.fill_area:
                ax.fill_between(
                    data[x],
                    data[col],
                    alpha=config.fill_alpha,
                    color=colors[i],
                )
        
        self._apply_config(ax, config)
        
        if len(y_cols) > 1:
            self._add_legend(ax, config)
        
        if config.tight_layout:
            self.plt.tight_layout()
        
        return fig
    
    def scatter(
        self,
        data: pd.DataFrame,
        x: str,
        y: str,
        size: Optional[Union[str, float]] = None,
        color: Optional[str] = None,
        config: Optional[PlotConfig] = None,
        **kwargs,
    ) -> Any:
        """Create a scatter plot."""
        config = self._merge_config(config)
        fig, ax = self._create_figure(config)
        
        # Determine sizes
        if size is None:
            sizes = config.scatter_size
        elif isinstance(size, str):
            # Map column to size range
            s_min, s_max = config.scatter_size_range
            s_data = data[size]
            sizes = s_min + (s_data - s_data.min()) / (s_data.max() - s_data.min()) * (s_max - s_min)
        else:
            sizes = size
        
        # Determine colors
        if color is None:
            c = self._get_colors(1, config)[0]
            scatter = ax.scatter(
                data[x],
                data[y],
                s=sizes,
                c=c,
                alpha=config.alpha,
                edgecolor=config.edge_color,
                linewidth=config.edge_width,
            )
        else:
            scatter = ax.scatter(
                data[x],
                data[y],
                s=sizes,
                c=data[color],
                cmap=config.colormap,
                alpha=config.alpha,
                edgecolor=config.edge_color,
                linewidth=config.edge_width,
            )
            
            if config.colorbar:
                cbar = self.plt.colorbar(scatter, ax=ax)
                if config.colorbar_label:
                    cbar.set_label(config.colorbar_label)
        
        self._apply_config(ax, config)
        
        if config.tight_layout:
            self.plt.tight_layout()
        
        return fig
    
    def heatmap(
        self,
        data: pd.DataFrame,
        config: Optional[PlotConfig] = None,
        **kwargs,
    ) -> Any:
        """Create a heatmap."""
        config = self._merge_config(config)
        fig, ax = self._create_figure(config)
        
        # Plot heatmap
        im = ax.imshow(
            data.values,
            cmap=config.colormap,
            aspect="auto",
        )
        
        # Set ticks
        ax.set_xticks(range(len(data.columns)))
        ax.set_yticks(range(len(data.index)))
        ax.set_xticklabels(data.columns, rotation=config.xtick_rotation)
        ax.set_yticklabels(data.index)
        
        # Add annotations
        if config.annotate:
            for i in range(len(data.index)):
                for j in range(len(data.columns)):
                    value = data.iloc[i, j]
                    text = config.annotation_format.format(value)
                    ax.text(
                        j, i, text,
                        ha="center",
                        va="center",
                        fontsize=config.annotation_font_size,
                        color="white" if value > data.values.mean() else "black",
                    )
        
        # Colorbar
        if config.colorbar:
            cbar = self.plt.colorbar(im, ax=ax)
            if config.colorbar_label:
                cbar.set_label(config.colorbar_label)
        
        self._apply_config(ax, config)
        
        if config.tight_layout:
            self.plt.tight_layout()
        
        return fig
    
    def network(
        self,
        nodes: pd.DataFrame,
        edges: pd.DataFrame,
        config: Optional[PlotConfig] = None,
        **kwargs,
    ) -> Any:
        """Create a network graph."""
        import networkx as nx
        
        config = self._merge_config(config)
        fig, ax = self._create_figure(config)
        
        # Create graph
        G = nx.Graph()
        
        # Add nodes
        for _, row in nodes.iterrows():
            G.add_node(row["id"], **row.to_dict())
        
        # Add edges
        for _, row in edges.iterrows():
            weight = row.get("weight", 1)
            G.add_edge(row["source"], row["target"], weight=weight)
        
        # Layout
        layout_funcs = {
            "spring": nx.spring_layout,
            "circular": nx.circular_layout,
            "kamada_kawai": nx.kamada_kawai_layout,
            "random": nx.random_layout,
        }
        pos = layout_funcs.get(config.layout, nx.spring_layout)(G)
        
        # Node sizes
        if config.node_size_by and config.node_size_by in nodes.columns:
            node_sizes = [
                nodes[nodes["id"] == n][config.node_size_by].values[0] * 10
                for n in G.nodes()
            ]
        else:
            node_sizes = config.node_size
        
        # Node colors
        if config.node_color_by and config.node_color_by in nodes.columns:
            node_colors = [
                nodes[nodes["id"] == n][config.node_color_by].values[0]
                for n in G.nodes()
            ]
        else:
            node_colors = self._get_colors(len(G.nodes()), config)
        
        # Edge widths
        if config.edge_width_by:
            edge_weights = [G[u][v].get(config.edge_width_by, 1) for u, v in G.edges()]
            max_weight = max(edge_weights) if edge_weights else 1
            edge_widths = [3 * w / max_weight for w in edge_weights]
        else:
            edge_widths = 1
        
        # Draw network
        nx.draw_networkx_edges(
            G, pos, ax=ax,
            width=edge_widths,
            alpha=config.edge_alpha,
        )
        
        nx.draw_networkx_nodes(
            G, pos, ax=ax,
            node_size=node_sizes,
            node_color=node_colors,
            alpha=config.alpha,
        )
        
        if config.show_labels:
            nx.draw_networkx_labels(
                G, pos, ax=ax,
                font_size=config.label_font_size,
            )
        
        ax.axis("off")
        
        if config.title:
            ax.set_title(
                config.title,
                fontsize=config.title_font_size,
                fontweight=config.title_font_weight,
            )
        
        if config.tight_layout:
            self.plt.tight_layout()
        
        return fig
    
    # =========================================================================
    # PHASE 2: DISTRIBUTION PLOTS
    # =========================================================================
    
    def histogram(
        self,
        data: pd.DataFrame,
        x: str,
        bins: int = 30,
        config: Optional[PlotConfig] = None,
        **kwargs,
    ) -> Any:
        """Create a histogram."""
        config = self._merge_config(config)
        fig, ax = self._create_figure(config)
        
        color = self._get_colors(1, config)[0]
        
        ax.hist(
            data[x].dropna(),
            bins=bins,
            color=color,
            alpha=config.alpha,
            edgecolor=config.edge_color if config.edge_color != "none" else "white",
            linewidth=config.edge_width if config.edge_width > 0 else 0.5,
        )
        
        self._apply_config(ax, config)
        
        if config.tight_layout:
            self.plt.tight_layout()
        
        return fig
    
    def boxplot(
        self,
        data: pd.DataFrame,
        x: Optional[str] = None,
        y: str = None,
        config: Optional[PlotConfig] = None,
        **kwargs,
    ) -> Any:
        """Create a box plot."""
        config = self._merge_config(config)
        fig, ax = self._create_figure(config)
        
        colors = self._get_colors(10, config)
        
        if x is not None:
            # Grouped boxplot
            groups = data[x].unique()
            box_data = [data[data[x] == g][y].dropna() for g in groups]
            
            bp = ax.boxplot(
                box_data,
                patch_artist=True,
                labels=groups,
            )
            
            for i, patch in enumerate(bp["boxes"]):
                patch.set_facecolor(colors[i % len(colors)])
                patch.set_alpha(config.alpha)
        else:
            # Single boxplot
            bp = ax.boxplot(
                [data[y].dropna()],
                patch_artist=True,
            )
            bp["boxes"][0].set_facecolor(colors[0])
            bp["boxes"][0].set_alpha(config.alpha)
        
        self._apply_config(ax, config)
        
        if config.tight_layout:
            self.plt.tight_layout()
        
        return fig
    
    def violinplot(
        self,
        data: pd.DataFrame,
        x: Optional[str] = None,
        y: str = None,
        config: Optional[PlotConfig] = None,
        **kwargs,
    ) -> Any:
        """Create a violin plot."""
        config = self._merge_config(config)
        fig, ax = self._create_figure(config)
        
        colors = self._get_colors(10, config)
        
        if x is not None:
            groups = data[x].unique()
            violin_data = [data[data[x] == g][y].dropna().values for g in groups]
            positions = range(1, len(groups) + 1)
            
            parts = ax.violinplot(violin_data, positions=positions, showmeans=True, showmedians=True)
            
            for i, pc in enumerate(parts["bodies"]):
                pc.set_facecolor(colors[i % len(colors)])
                pc.set_alpha(config.alpha)
            
            ax.set_xticks(positions)
            ax.set_xticklabels(groups)
        else:
            parts = ax.violinplot([data[y].dropna().values], showmeans=True, showmedians=True)
            parts["bodies"][0].set_facecolor(colors[0])
            parts["bodies"][0].set_alpha(config.alpha)
        
        self._apply_config(ax, config)
        
        if config.tight_layout:
            self.plt.tight_layout()
        
        return fig
    
    # =========================================================================
    # PHASE 2: AREA PLOTS
    # =========================================================================
    
    def area(
        self,
        data: pd.DataFrame,
        x: str,
        y: Union[str, List[str]],
        stacked: bool = False,
        config: Optional[PlotConfig] = None,
        **kwargs,
    ) -> Any:
        """Create an area chart."""
        config = self._merge_config(config)
        fig, ax = self._create_figure(config)
        
        y_cols = [y] if isinstance(y, str) else y
        colors = self._get_colors(len(y_cols), config)
        
        if stacked:
            ax.stackplot(
                data[x],
                [data[col] for col in y_cols],
                labels=y_cols,
                colors=colors,
                alpha=config.alpha,
            )
        else:
            for i, col in enumerate(y_cols):
                ax.fill_between(
                    data[x],
                    data[col],
                    alpha=config.fill_alpha,
                    color=colors[i],
                    label=col,
                )
                ax.plot(data[x], data[col], color=colors[i], linewidth=config.line_width)
        
        self._apply_config(ax, config)
        
        if len(y_cols) > 1:
            self._add_legend(ax, config)
        
        if config.tight_layout:
            self.plt.tight_layout()
        
        return fig
    
    # =========================================================================
    # PHASE 2: GROUPED/STACKED BAR PLOTS
    # =========================================================================
    
    def grouped_bar(
        self,
        data: pd.DataFrame,
        x: str,
        y: List[str],
        config: Optional[PlotConfig] = None,
        **kwargs,
    ) -> Any:
        """Create a grouped bar chart."""
        config = self._merge_config(config)
        fig, ax = self._create_figure(config)
        
        n_groups = len(data)
        n_bars = len(y)
        colors = self._get_colors(n_bars, config)
        
        bar_width = config.bar_width / n_bars
        positions = np.arange(n_groups)
        
        for i, col in enumerate(y):
            offset = (i - n_bars / 2 + 0.5) * bar_width
            ax.bar(
                positions + offset,
                data[col],
                bar_width,
                label=col,
                color=colors[i],
                alpha=config.alpha,
            )
        
        ax.set_xticks(positions)
        ax.set_xticklabels(data[x])
        
        self._apply_config(ax, config)
        self._add_legend(ax, config)
        
        if config.tight_layout:
            self.plt.tight_layout()
        
        return fig
    
    def stacked_bar(
        self,
        data: pd.DataFrame,
        x: str,
        y: List[str],
        horizontal: bool = False,
        config: Optional[PlotConfig] = None,
        **kwargs,
    ) -> Any:
        """Create a stacked bar chart."""
        config = self._merge_config(config)
        fig, ax = self._create_figure(config)
        
        colors = self._get_colors(len(y), config)
        
        if horizontal:
            bottom = np.zeros(len(data))
            for i, col in enumerate(y):
                ax.barh(
                    data[x],
                    data[col],
                    left=bottom,
                    label=col,
                    color=colors[i],
                    alpha=config.alpha,
                    height=config.bar_width,
                )
                bottom += data[col].values
        else:
            bottom = np.zeros(len(data))
            for i, col in enumerate(y):
                ax.bar(
                    data[x],
                    data[col],
                    bottom=bottom,
                    label=col,
                    color=colors[i],
                    alpha=config.alpha,
                    width=config.bar_width,
                )
                bottom += data[col].values
        
        self._apply_config(ax, config)
        self._add_legend(ax, config)
        
        if config.tight_layout:
            self.plt.tight_layout()
        
        return fig
    
    # =========================================================================
    # PHASE 2: COMPARISON PLOTS
    # =========================================================================
    
    def dumbbell(
        self,
        data: pd.DataFrame,
        y: str,
        x_start: str,
        x_end: str,
        config: Optional[PlotConfig] = None,
        **kwargs,
    ) -> Any:
        """Create a dumbbell chart."""
        config = self._merge_config(config)
        fig, ax = self._create_figure(config)
        
        colors = self._get_colors(2, config)
        
        y_positions = range(len(data))
        
        # Draw lines
        for i, (_, row) in enumerate(data.iterrows()):
            ax.plot(
                [row[x_start], row[x_end]],
                [i, i],
                color="gray",
                linewidth=config.line_width,
                alpha=0.5,
            )
        
        # Draw start points
        ax.scatter(
            data[x_start],
            y_positions,
            color=colors[0],
            s=config.marker_size * 20,
            zorder=5,
            label=x_start,
        )
        
        # Draw end points
        ax.scatter(
            data[x_end],
            y_positions,
            color=colors[1],
            s=config.marker_size * 20,
            zorder=5,
            label=x_end,
        )
        
        ax.set_yticks(y_positions)
        ax.set_yticklabels(data[y])
        
        self._apply_config(ax, config)
        self._add_legend(ax, config)
        
        if config.tight_layout:
            self.plt.tight_layout()
        
        return fig
    
    def lollipop(
        self,
        data: pd.DataFrame,
        x: str,
        y: str,
        horizontal: bool = True,
        config: Optional[PlotConfig] = None,
        **kwargs,
    ) -> Any:
        """Create a lollipop chart."""
        config = self._merge_config(config)
        fig, ax = self._create_figure(config)
        
        color = self._get_colors(1, config)[0]
        
        if horizontal:
            positions = range(len(data))
            
            # Draw stems
            ax.hlines(
                y=positions,
                xmin=0,
                xmax=data[x],
                color=color,
                linewidth=config.line_width,
                alpha=0.7,
            )
            
            # Draw dots
            ax.scatter(
                data[x],
                positions,
                color=color,
                s=config.marker_size * 15,
                zorder=5,
            )
            
            ax.set_yticks(positions)
            ax.set_yticklabels(data[y])
        else:
            # Vertical lollipop
            ax.vlines(
                x=data[y],
                ymin=0,
                ymax=data[x],
                color=color,
                linewidth=config.line_width,
                alpha=0.7,
            )
            
            ax.scatter(
                data[y],
                data[x],
                color=color,
                s=config.marker_size * 15,
                zorder=5,
            )
        
        self._apply_config(ax, config)
        
        if config.tight_layout:
            self.plt.tight_layout()
        
        return fig
    
    # =========================================================================
    # PHASE 2: BUBBLE CHART
    # =========================================================================
    
    def bubble(
        self,
        data: pd.DataFrame,
        x: str,
        y: str,
        size: str,
        color: Optional[str] = None,
        label: Optional[str] = None,
        config: Optional[PlotConfig] = None,
        **kwargs,
    ) -> Any:
        """Create a bubble chart."""
        config = self._merge_config(config)
        fig, ax = self._create_figure(config)
        
        # Scale sizes
        s_min, s_max = config.scatter_size_range
        s_data = data[size]
        sizes = s_min + (s_data - s_data.min()) / (s_data.max() - s_data.min() + 1e-10) * (s_max - s_min)
        
        # Colors
        if color:
            scatter = ax.scatter(
                data[x],
                data[y],
                s=sizes,
                c=data[color],
                cmap=config.colormap,
                alpha=config.alpha,
                edgecolor="white",
                linewidth=0.5,
            )
            if config.colorbar:
                cbar = self.plt.colorbar(scatter, ax=ax)
                if config.colorbar_label:
                    cbar.set_label(config.colorbar_label)
        else:
            colors = self._get_colors(1, config)[0]
            ax.scatter(
                data[x],
                data[y],
                s=sizes,
                c=colors,
                alpha=config.alpha,
                edgecolor="white",
                linewidth=0.5,
            )
        
        # Labels
        if label and config.show_labels:
            for _, row in data.iterrows():
                ax.annotate(
                    str(row[label])[:15],
                    (row[x], row[y]),
                    fontsize=config.label_font_size,
                    ha="center",
                    va="bottom",
                )
        
        self._apply_config(ax, config)
        
        if config.tight_layout:
            self.plt.tight_layout()
        
        return fig
    
    # =========================================================================
    # PHASE 2: DONUT CHART
    # =========================================================================
    
    def donut(
        self,
        data: pd.DataFrame,
        values: str,
        labels: str,
        hole_size: float = 0.4,
        config: Optional[PlotConfig] = None,
        **kwargs,
    ) -> Any:
        """Create a donut chart."""
        config = self._merge_config(config)
        fig, ax = self._create_figure(config)
        
        colors = self._get_colors(len(data), config)
        
        wedges, texts, autotexts = ax.pie(
            data[values],
            labels=data[labels] if config.show_labels else None,
            colors=colors,
            autopct=lambda p: f"{p:.1f}%" if p > 5 else "",
            pctdistance=0.75,
            startangle=90,
            wedgeprops=dict(width=1 - hole_size, edgecolor="white"),
        )
        
        # Center circle for donut effect
        centre_circle = self.plt.Circle((0, 0), hole_size, fc=config.background_color)
        ax.add_patch(centre_circle)
        
        ax.axis("equal")
        
        if config.title:
            ax.set_title(
                config.title,
                fontsize=config.title_font_size,
                fontweight=config.title_font_weight,
            )
        
        if config.tight_layout:
            self.plt.tight_layout()
        
        return fig
    
    # =========================================================================
    # PHASE 2: TREEMAP
    # =========================================================================
    
    def treemap(
        self,
        data: pd.DataFrame,
        values: str,
        labels: str,
        parents: Optional[str] = None,
        config: Optional[PlotConfig] = None,
        **kwargs,
    ) -> Any:
        """Create a treemap."""
        config = self._merge_config(config)
        fig, ax = self._create_figure(config)
        
        try:
            import squarify
        except ImportError:
            raise ImportError("squarify required for treemaps: pip install squarify")
        
        colors = self._get_colors(len(data), config)
        
        # Normalize values
        norm_values = data[values] / data[values].sum() * 100
        
        squarify.plot(
            sizes=norm_values,
            label=data[labels] if config.show_labels else None,
            color=colors,
            alpha=config.alpha,
            ax=ax,
            text_kwargs={"fontsize": config.label_font_size},
        )
        
        ax.axis("off")
        
        if config.title:
            ax.set_title(
                config.title,
                fontsize=config.title_font_size,
                fontweight=config.title_font_weight,
            )
        
        if config.tight_layout:
            self.plt.tight_layout()
        
        return fig
    
    # =========================================================================
    # PHASE 2: WORDCLOUD
    # =========================================================================
    
    def wordcloud(
        self,
        data: pd.DataFrame,
        text: str,
        weight: Optional[str] = None,
        config: Optional[PlotConfig] = None,
        **kwargs,
    ) -> Any:
        """Create a word cloud."""
        config = self._merge_config(config)
        fig, ax = self._create_figure(config)
        
        try:
            from wordcloud import WordCloud
        except ImportError:
            raise ImportError("wordcloud required: pip install wordcloud")
        
        # Build frequency dict
        if weight:
            freq = dict(zip(data[text], data[weight]))
        else:
            freq = dict(zip(data[text], [1] * len(data)))
        
        wc = WordCloud(
            width=config.width,
            height=config.height,
            background_color=config.background_color,
            colormap=config.colormap,
            max_words=100,
        ).generate_from_frequencies(freq)
        
        ax.imshow(wc, interpolation="bilinear")
        ax.axis("off")
        
        if config.title:
            ax.set_title(
                config.title,
                fontsize=config.title_font_size,
                fontweight=config.title_font_weight,
            )
        
        if config.tight_layout:
            self.plt.tight_layout()
        
        return fig
    
    # =========================================================================
    # PHASE 2: RADAR CHART
    # =========================================================================
    
    def radar(
        self,
        data: pd.DataFrame,
        categories: List[str],
        values: Union[str, List[str]],
        config: Optional[PlotConfig] = None,
        **kwargs,
    ) -> Any:
        """Create a radar/spider chart."""
        config = self._merge_config(config)
        
        # Number of variables
        num_vars = len(categories)
        angles = np.linspace(0, 2 * np.pi, num_vars, endpoint=False).tolist()
        angles += angles[:1]  # Complete the circle
        
        fig, ax = self.plt.subplots(
            figsize=(config.width / config.dpi, config.height / config.dpi),
            subplot_kw=dict(polar=True),
        )
        
        value_cols = [values] if isinstance(values, str) else values
        colors = self._get_colors(len(value_cols), config)
        
        for i, col in enumerate(value_cols):
            if col in data.columns:
                vals = data[col].values.tolist()
            else:
                # Assume col is a row index
                vals = data.loc[col, categories].values.tolist()
            
            vals += vals[:1]  # Complete the circle
            
            ax.plot(angles, vals, color=colors[i], linewidth=config.line_width, label=col)
            ax.fill(angles, vals, color=colors[i], alpha=config.fill_alpha)
        
        ax.set_xticks(angles[:-1])
        ax.set_xticklabels(categories, fontsize=config.xtick_font_size)
        
        if config.title:
            ax.set_title(
                config.title,
                fontsize=config.title_font_size,
                fontweight=config.title_font_weight,
            )
        
        if len(value_cols) > 1 and config.show_legend:
            ax.legend(loc="upper right", bbox_to_anchor=(1.3, 1.0))
        
        if config.tight_layout:
            self.plt.tight_layout()
        
        return fig
    
    # =========================================================================
    # SAVE AND SHOW
    # =========================================================================
    
    def save(
        self,
        figure: Any,
        filename: str,
        config: Optional[PlotConfig] = None,
    ) -> List[str]:
        """Save figure to file(s)."""
        config = config or self.config
        
        saved_files = []
        for fmt in config.save_format:
            path = f"{filename}.{fmt}"
            figure.savefig(
                path,
                dpi=config.export_dpi,
                bbox_inches="tight" if config.tight_layout else None,
                transparent=config.transparent,
                facecolor=figure.get_facecolor(),
            )
            saved_files.append(path)
        
        print(f"Plot saved to {filename}.{config.save_format[0]} (and {', '.join(config.save_format[1:])})")
        
        return saved_files
    
    def show(self, figure: Any) -> None:
        """Display figure."""
        self.plt.show()



"""
Dynamic Topic Modeling Panel
============================

Panel for Sequential and Dynamic Topic Modeling analysis.
Tracks how topics evolve over time.
"""

import tkinter as tk
from tkinter import ttk, messagebox, filedialog
from typing import Optional, Dict, Any, List
import pandas as pd
import numpy as np
import threading

from biblium.gui.panels.base import BasePanel
from biblium.gui.config import FONTS, get_theme
from biblium.gui.widgets.tables import DataTable
from biblium.gui.widgets.plots import PlotFrame
from biblium.gui.widgets.forms import LabeledEntry, LabeledCombobox, LabeledSpinbox
from biblium.gui.widgets.cards import Card, StatsCard, CardGrid
from biblium.gui.widgets.buttons import ActionButton

try:
    import matplotlib.pyplot as plt
    import seaborn as sns
    HAS_MATPLOTLIB = True
except ImportError:
    HAS_MATPLOTLIB = False


class DynamicTopicsPanel(BasePanel):
    """Panel for Dynamic and Sequential Topic Modeling."""
    
    title = "Dynamic Topics"
    icon = "🔄"
    description = "Track topic evolution over time"
    requires_data = True
    
    MODELS = ["LDA", "NMF", "LSA"]
    METHODS = ["Dynamic (DTM)", "Sequential (STM)"]
    TEXT_COLUMNS = ["Processed Abstract", "Abstract", "Processed Title", "Title"]
    
    def __init__(self, parent, theme: str = "light", bib=None, **kwargs):
        self._result = None
        self._method = "DTM"
        super().__init__(parent, theme=theme, bib=bib, **kwargs)
    
    def _create_options(self):
        """Create the options panel."""
        self._add_title()
        
        # Method Selection
        method_card = Card(self.options_content, title="🔄 Method", theme=self.theme_name)
        method_card.pack(fill=tk.X, padx=8, pady=8)
        
        self.method_combo = LabeledCombobox(
            method_card.content, label="Analysis Type:",
            values=self.METHODS, default="Dynamic (DTM)",
            theme=self.theme_name, label_width=14,
        )
        self.method_combo.pack(fill=tk.X, pady=4)
        self.method_combo.combobox.bind("<<ComboboxSelected>>", self._on_method_change)
        
        # Text Selection
        text_card = Card(self.options_content, title="📄 Text Selection", theme=self.theme_name)
        text_card.pack(fill=tk.X, padx=8, pady=8)
        
        self.text_combo = LabeledCombobox(
            text_card.content, label="Text Column:",
            values=self.TEXT_COLUMNS, default="Processed Abstract",
            theme=self.theme_name, label_width=14,
        )
        self.text_combo.pack(fill=tk.X, pady=4)
        
        self.time_combo = LabeledCombobox(
            text_card.content, label="Time Column:",
            values=["Year", "Publication Year", "PY"],
            default="Year", theme=self.theme_name, label_width=14,
        )
        self.time_combo.pack(fill=tk.X, pady=4)
        
        # Model Settings
        model_card = Card(self.options_content, title="🔬 Model Settings", theme=self.theme_name)
        model_card.pack(fill=tk.X, padx=8, pady=8)
        
        self.model_combo = LabeledCombobox(
            model_card.content, label="Model Type:",
            values=self.MODELS, default="LDA",
            theme=self.theme_name, label_width=14,
        )
        self.model_combo.pack(fill=tk.X, pady=4)
        
        self.n_topics_spin = LabeledSpinbox(
            model_card.content, label="Number of Topics:",
            from_=2, to=20, default=5,
            theme=self.theme_name, label_width=16,
        )
        self.n_topics_spin.pack(fill=tk.X, pady=4)
        
        self.n_slices_spin = LabeledSpinbox(
            model_card.content, label="Time Slices:",
            from_=3, to=20, default=5,
            theme=self.theme_name, label_width=16,
        )
        self.n_slices_spin.pack(fill=tk.X, pady=4)
        
        # Advanced
        advanced_card = Card(self.options_content, title="⚙️ Advanced", theme=self.theme_name)
        advanced_card.pack(fill=tk.X, padx=8, pady=8)
        
        self.max_features_spin = LabeledSpinbox(
            advanced_card.content, label="Max Features:",
            from_=500, to=10000, default=3000,
            theme=self.theme_name, label_width=14,
        )
        self.max_features_spin.pack(fill=tk.X, pady=4)
        
        self.variance_frame = tk.Frame(advanced_card.content, bg=self.theme["bg_card"])
        self.variance_frame.pack(fill=tk.X, pady=4)
        
        tk.Label(
            self.variance_frame, text="Smoothing:",
            font=FONTS.get_font("body"), bg=self.theme["bg_card"],
            fg=self.theme["text_primary"], width=14, anchor="w"
        ).pack(side=tk.LEFT)
        
        self.variance_scale = tk.Scale(
            self.variance_frame, from_=0.01, to=0.5,
            resolution=0.01, orient=tk.HORIZONTAL,
            bg=self.theme["bg_card"], fg=self.theme["text_primary"],
            highlightthickness=0, length=150
        )
        self.variance_scale.set(0.1)
        self.variance_scale.pack(side=tk.LEFT, fill=tk.X, expand=True)
        
        # Run Button
        # LLM Configuration (v2.11)
        self._create_llm_config_card()
        
        btn_frame = tk.Frame(self.options_content, bg=self.theme["bg_secondary"])
        btn_frame.pack(fill=tk.X, padx=8, pady=16)
        
        ActionButton(
            btn_frame, text="Run Analysis", icon="🔄",
            command=self._run_analysis, theme=self.theme_name,
        ).pack(fill=tk.X)
        
        # Export
        export_card = Card(self.options_content, title="📥 Export", theme=self.theme_name)
        export_card.pack(fill=tk.X, padx=8, pady=8)
        
        tk.Button(
            export_card.content, text="Export Results to Excel",
            command=self._export_results,
            bg=self.theme["bg_secondary"], fg=self.theme["text_primary"],
            font=FONTS.get_font("body"), relief=tk.FLAT, cursor="hand2"
        ).pack(fill=tk.X, pady=4)
    
    def _on_method_change(self, event=None):
        """Handle method selection change."""
        is_dtm = "Dynamic" in self.method_combo.get()
        if is_dtm:
            self.n_slices_spin.pack(fill=tk.X, pady=4)
            self.variance_frame.pack(fill=tk.X, pady=4)
        else:
            self.n_slices_spin.pack_forget()
            self.variance_frame.pack_forget()
    
    def _create_results(self):
        """Create results panel."""
        super()._create_results()
        
        # Create notebook with permanent Info tab
        self.results_notebook = ttk.Notebook(self.results_content)
        self.results_notebook.pack(fill=tk.BOTH, expand=True)
        
        # Results tab
        self.results_tab = tk.Frame(self.results_notebook, bg=self.theme["bg_card"])
        self.results_notebook.add(self.results_tab, text="📊 Results")
        
        # Info tab (always visible)
        self.info_frame = tk.Frame(self.results_notebook, bg=self.theme["bg_card"])
        self.results_notebook.add(self.info_frame, text="ℹ️ Info")
        self._create_info_content(self.info_frame)
        
        # Show placeholder in results tab
        self._show_results_placeholder()
    
    def _show_results_placeholder(self):
        """Show placeholder in results tab."""
        # Check if results_tab still exists
        if not hasattr(self, 'results_tab'):
            return
        try:
            if not self.results_tab.winfo_exists():
                return
        except Exception:
            return
        
        for widget in self.results_tab.winfo_children():
            widget.destroy()
        tk.Label(
            self.results_tab,
            text="Click 'Run' to see results here.\nSee Info tab for documentation.",
            font=FONTS.get_font("body"),
            bg=self.theme["bg_card"],
            fg=self.theme["text_muted"],
            justify=tk.CENTER,
        ).pack(expand=True)

    def _show_placeholder(self):
        """Show detailed placeholder with instructions."""
        # Check if results_tab still exists
        if not hasattr(self, 'results_tab'):
            return
        try:
            if not self.results_tab.winfo_exists():
                return
        except Exception:
            return
        
        for widget in self.results_tab.winfo_children():
            widget.destroy()
        
        msg = (
            "📅 Dynamic Topic Modeling\n\n"
            "Discover how topics evolve over time.\n\n"
            "Features:\n"
            "• Structural Topic Models (STM)\n"
            "• Dynamic Topic Models (DTM)\n"
            "• Topic prevalence over time\n"
            "• Emerging and declining topics\n"
            "\n"
            "Captures topic evolution and research fronts.\n\n"
            "Steps:\n"
            "1. Load dataset with abstracts and years\n"
            "2. Select model type\n"
            "3. Set number of topics\n"
            "4. Click 'Run Analysis'\n"
        )
        
        tk.Label(
            self.results_tab,
            text=msg,
            font=FONTS.get_font("body"),
            bg=self.theme["bg_card"],
            fg=self.theme["text_muted"],
            justify=tk.CENTER,
        ).pack(expand=True)

    def _run_analysis(self):
        """Run analysis."""
        if not self.bib:
            messagebox.showwarning("No Data", "Please load a dataset first.")
            return
        
        method = self.method_combo.get()
        is_dtm = "Dynamic" in method
        self._method = "DTM" if is_dtm else "STM"
        
        text_column = self.text_combo.get()
        time_column = self.time_combo.get()
        
        # Check columns
        if text_column not in self.bib.df.columns:
            for fb in ["Abstract", "Processed Title", "Title"]:
                if fb in self.bib.df.columns:
                    text_column = fb
                    break
        
        if time_column not in self.bib.df.columns:
            for fb in ["Year", "Publication Year", "PY"]:
                if fb in self.bib.df.columns:
                    time_column = fb
                    break
        
        if time_column not in self.bib.df.columns:
            messagebox.showerror("Error", "No time/year column found.")
            return
        
        self._show_loading(f"Running {'Dynamic' if is_dtm else 'Sequential'} Topic Modeling...")
        
        def do_analysis():
            try:
                from biblium import utilsbib
                
                if is_dtm:
                    result = utilsbib.dynamic_topic_modeling(
                        df=self.bib.df,
                        text_column=text_column,
                        time_column=time_column,
                        n_topics=self.n_topics_spin.get(),
                        n_time_slices=self.n_slices_spin.get(),
                        model_type=self.model_combo.get(),
                        max_features=self.max_features_spin.get(),
                        chain_variance=self.variance_scale.get(),
                    )
                else:
                    result = utilsbib.sequential_topic_modeling(
                        df=self.bib.df,
                        text_column=text_column,
                        time_column=time_column,
                        n_topics=self.n_topics_spin.get(),
                        model_type=self.model_combo.get(),
                        max_features=self.max_features_spin.get(),
                    )
                
                self._result = result
                self.after(0, lambda: self._on_success(result))
                
            except Exception as e:
                import traceback
                self.after(0, lambda: self._show_error(f"{e}\n\n{traceback.format_exc()}"))
        
        threading.Thread(target=do_analysis, daemon=True).start()
    
    def _on_success(self, result):
        """Display results."""
        self._stop_active_spinners()
        self._safe_clear_results()
        
        is_dtm = self._method == "DTM"
        
        if is_dtm and result.get("n_topics", 0) == 0:
            self._show_error("No topics found.")
            return
        if not is_dtm and not result.get("period_topics"):
            self._show_error("No topics found.")
            return
        
        notebook = ttk.Notebook(self.results_tab)
        notebook.pack(fill=tk.BOTH, expand=True)
        
        # Summary
        summary_frame = tk.Frame(notebook, bg=self.theme["bg_card"])
        notebook.add(summary_frame, text="Summary")
        self._create_summary_tab(summary_frame, result, is_dtm)
        
        # Prevalence
        prevalence_frame = tk.Frame(notebook, bg=self.theme["bg_card"])
        notebook.add(prevalence_frame, text="Prevalence")
        self._create_prevalence_tab(prevalence_frame, result, is_dtm)
        
        # Streams (DTM)
        if is_dtm and HAS_MATPLOTLIB:
            streams_frame = tk.Frame(notebook, bg=self.theme["bg_card"])
            notebook.add(streams_frame, text="Topic Streams")
            self._create_streams_tab(streams_frame, result)
        
        # Word Evolution (DTM)
        if is_dtm:
            word_frame = tk.Frame(notebook, bg=self.theme["bg_card"])
            notebook.add(word_frame, text="Word Evolution")
            self._create_word_evolution_tab(word_frame, result)
        
        # Global Topics (DTM) / Alignment (STM)
        if is_dtm:
            global_frame = tk.Frame(notebook, bg=self.theme["bg_card"])
            notebook.add(global_frame, text="Global Topics")
            self._create_global_topics_tab(global_frame, result)
        else:
            evo_frame = tk.Frame(notebook, bg=self.theme["bg_card"])
            notebook.add(evo_frame, text="Topic Alignment")
            self._create_evolution_tab(evo_frame, result)
            
            period_frame = tk.Frame(notebook, bg=self.theme["bg_card"])
            notebook.add(period_frame, text="Period Topics")
            self._create_period_topics_tab(period_frame, result)
        
        # Term Evolution tab (for both DTM and STM)
        if HAS_MATPLOTLIB:
            term_frame = tk.Frame(notebook, bg=self.theme["bg_card"])
            notebook.add(term_frame, text="Term Evolution")

            

            # Info tab

            info_frame = tk.Frame(notebook, bg=self.theme["bg_card"])

            notebook.add(info_frame, text="ℹ️ Info")

            self._create_info_content(info_frame)
            self._create_term_evolution_tab(term_frame)
        
        n_topics = result.get("n_topics", len(result.get("period_topics", {})))
        messagebox.showinfo("Complete", f"Analysis complete with {n_topics} topics.")
    
    def _create_summary_tab(self, parent, result, is_dtm):
        """Create summary tab."""
        tk.Label(
            parent, text=f"{'Dynamic' if is_dtm else 'Sequential'} Topic Modeling Summary",
            font=FONTS.get_font("heading2"),
            bg=self.theme["bg_card"], fg=self.theme["text_primary"]
        ).pack(pady=(16, 8))
        
        grid = CardGrid(parent, columns=4, theme=self.theme_name)
        grid.pack(fill=tk.X, padx=16, pady=8)
        
        n_topics = result.get("n_topics", 0)
        
        if is_dtm:
            n_slices = result.get("n_time_slices", 0)
            grid.add_card(StatsCard(grid, "Topics", f"{n_topics}", "📚", self.theme_name, accent=True))
            grid.add_card(StatsCard(grid, "Time Slices", f"{n_slices}", "📅", self.theme_name))
            grid.add_card(StatsCard(grid, "Method", "DTM", "🔄", self.theme_name))
            grid.add_card(StatsCard(grid, "Model", self.model_combo.get(), "🔬", self.theme_name))
            
            time_info = result.get("time_slice_info", pd.DataFrame())
            if not time_info.empty:
                tk.Label(parent, text="Time Slices", font=FONTS.get_font("heading3"),
                        bg=self.theme["bg_card"], fg=self.theme["text_primary"]).pack(pady=(16, 8))
                table = DataTable(parent, theme=self.theme_name)
                table.pack(fill=tk.BOTH, expand=True, padx=16, pady=8)
                table.set_data(time_info)
        else:
            n_periods = len(result.get("periods", []))
            grid.add_card(StatsCard(grid, "Topics/Period", f"{n_topics}", "📚", self.theme_name, accent=True))
            grid.add_card(StatsCard(grid, "Periods", f"{n_periods}", "📅", self.theme_name))
            grid.add_card(StatsCard(grid, "Method", "STM", "🔄", self.theme_name))
            grid.add_card(StatsCard(grid, "Model", self.model_combo.get(), "🔬", self.theme_name))
            
            period_stats = result.get("period_stats", pd.DataFrame())
            if not period_stats.empty:
                tk.Label(parent, text="Period Statistics", font=FONTS.get_font("heading3"),
                        bg=self.theme["bg_card"], fg=self.theme["text_primary"]).pack(pady=(16, 8))
                table = DataTable(parent, theme=self.theme_name)
                table.pack(fill=tk.BOTH, expand=True, padx=16, pady=8)
                table.set_data(period_stats)
    
    def _create_prevalence_tab(self, parent, result, is_dtm):
        """Create prevalence plot."""
        if not HAS_MATPLOTLIB:
            tk.Label(parent, text="Matplotlib required.", font=FONTS.get_font("body"),
                    bg=self.theme["bg_card"], fg=self.theme["text_muted"]).pack(expand=True)
            return
        
        prevalence_df = result.get("topic_prevalence_evolution" if is_dtm else "topic_prevalence", pd.DataFrame())
        time_col = "Time_Slice" if is_dtm else "Period"
        
        if prevalence_df.empty:
            tk.Label(parent, text="No prevalence data.", font=FONTS.get_font("body"),
                    bg=self.theme["bg_card"], fg=self.theme["text_muted"]).pack(expand=True)
            return
        
        control_frame = tk.Frame(parent, bg=self.theme["bg_card"])
        control_frame.pack(fill=tk.X, padx=8, pady=8)
        
        self._stacked_var = tk.BooleanVar(value=True)
        tk.Checkbutton(
            control_frame, text="Stacked", variable=self._stacked_var,
            bg=self.theme["bg_card"], fg=self.theme["text_primary"],
            command=lambda: self._update_prevalence(prevalence_df, time_col)
        ).pack(side=tk.LEFT, padx=8)
        
        self._prev_plot = PlotFrame(parent, theme=self.theme_name, show_toolbar=True, show_ai_button=True)
        self._prev_plot.pack(fill=tk.BOTH, expand=True)
        self._update_prevalence(prevalence_df, time_col)
    
    def _update_prevalence(self, df, time_col):
        """Update prevalence plot."""
        fig, ax = self._prev_plot.get_figure()
        ax.clear()
        
        pivot = df.pivot(index=time_col, columns='Topic', values='Prevalence').fillna(0)
        topics = pivot.columns.tolist()
        colors = plt.cm.get_cmap('tab10')(np.linspace(0, 1, len(topics)))
        
        if self._stacked_var.get():
            ax.stackplot(pivot.index, pivot.T.values, labels=topics, colors=colors, alpha=0.8)
        else:
            for i, t in enumerate(topics):
                ax.plot(pivot.index, pivot[t], label=t, color=colors[i], linewidth=2, marker='o')
        
        ax.set_xlabel("Time Period")
        ax.set_ylabel("Prevalence")
        ax.set_title("Topic Prevalence Over Time")
        ax.legend(bbox_to_anchor=(1.02, 1), loc='upper left', fontsize=8)
        ax.grid(False)
        ax.spines['top'].set_visible(False)
        ax.spines['right'].set_visible(False)
        fig.tight_layout()
        self._prev_plot.refresh()
    
    def _create_streams_tab(self, parent, result):
        """Create topic streams."""
        df = result.get("topic_prevalence_evolution", pd.DataFrame())
        if df.empty:
            tk.Label(parent, text="No data.", font=FONTS.get_font("body"),
                    bg=self.theme["bg_card"], fg=self.theme["text_muted"]).pack(expand=True)
            return
        
        plot = PlotFrame(parent, theme=self.theme_name, show_toolbar=True, show_ai_button=True)
        plot.pack(fill=tk.BOTH, expand=True)
        fig, ax = plot.get_figure()
        
        pivot = df.pivot(index='Time_Slice', columns='Topic', values='Prevalence').fillna(0)
        pivot = pivot.div(pivot.sum(axis=1), axis=0)
        
        topics = pivot.columns.tolist()
        x = pivot.index.values
        colors = plt.cm.get_cmap('tab10')(np.linspace(0, 1, len(topics)))
        
        y_stack = np.row_stack([pivot[t].values for t in topics])
        baseline = -y_stack.sum(axis=0) / 2
        y_bottom = baseline.copy()
        
        for i, topic in enumerate(topics):
            y_top = y_bottom + pivot[topic].values
            ax.fill_between(x, y_bottom, y_top, label=topic, color=colors[i], alpha=0.8)
            y_bottom = y_top
        
        ax.set_xlabel("Time Slice")
        ax.set_ylabel("Relative Prevalence")
        ax.set_title("Topic Streams Over Time")
        ax.legend(bbox_to_anchor=(1.02, 1), loc='upper left', fontsize=8)
        ax.grid(False)
        fig.tight_layout()
        plot.refresh()
    
    def _create_word_evolution_tab(self, parent, result):
        """Create word evolution tab."""
        evo = result.get("topic_word_evolution", {})
        if not evo:
            tk.Label(parent, text="No data.", font=FONTS.get_font("body"),
                    bg=self.theme["bg_card"], fg=self.theme["text_muted"]).pack(expand=True)
            return
        
        control = tk.Frame(parent, bg=self.theme["bg_card"])
        control.pack(fill=tk.X, padx=8, pady=8)
        
        tk.Label(control, text="Topic:", font=FONTS.get_font("body"),
                bg=self.theme["bg_card"], fg=self.theme["text_primary"]).pack(side=tk.LEFT)
        
        topics = list(evo.keys())
        self._evo_var = tk.StringVar(value=topics[0] if topics else "")
        combo = ttk.Combobox(control, textvariable=self._evo_var, values=topics, state="readonly", width=15)
        combo.pack(side=tk.LEFT, padx=4)
        
        self._evo_table = DataTable(parent, theme=self.theme_name)
        self._evo_table.pack(fill=tk.BOTH, expand=True, padx=8, pady=8)
        self._evo_data = evo
        
        def update(*args):
            t = self._evo_var.get()
            if t in self._evo_data:
                self._evo_table.set_data(self._evo_data[t])
        
        combo.bind("<<ComboboxSelected>>", update)
        update()
    
    def _create_global_topics_tab(self, parent, result):
        """Create global topics tab."""
        df = result.get("global_topics", pd.DataFrame())
        if df.empty:
            tk.Label(parent, text="No data.", font=FONTS.get_font("body"),
                    bg=self.theme["bg_card"], fg=self.theme["text_muted"]).pack(expand=True)
            return
        
        tk.Label(parent, text="Global Topic-Word Weights", font=FONTS.get_font("heading3"),
                bg=self.theme["bg_card"], fg=self.theme["text_primary"]).pack(pady=(16, 8))
        
        table = DataTable(parent, theme=self.theme_name)
        table.pack(fill=tk.BOTH, expand=True, padx=8, pady=8)
        table.set_data(df)
    
    def _create_evolution_tab(self, parent, result):
        """Create topic alignment tab (STM)."""
        df = result.get("topic_evolution", pd.DataFrame())
        if df.empty:
            tk.Label(parent, text="No data.", font=FONTS.get_font("body"),
                    bg=self.theme["bg_card"], fg=self.theme["text_muted"]).pack(expand=True)
            return
        
        tk.Label(parent, text="Topic Alignment Across Periods", font=FONTS.get_font("heading3"),
                bg=self.theme["bg_card"], fg=self.theme["text_primary"]).pack(pady=(16, 8))
        
        table = DataTable(parent, theme=self.theme_name)
        table.pack(fill=tk.BOTH, expand=True, padx=8, pady=8)
        table.set_data(df)
    
    def _create_period_topics_tab(self, parent, result):
        """Create period topics tab (STM)."""
        period_topics = result.get("period_topics", {})
        if not period_topics:
            tk.Label(parent, text="No data.", font=FONTS.get_font("body"),
                    bg=self.theme["bg_card"], fg=self.theme["text_muted"]).pack(expand=True)
            return
        
        control = tk.Frame(parent, bg=self.theme["bg_card"])
        control.pack(fill=tk.X, padx=8, pady=8)
        
        tk.Label(control, text="Period:", font=FONTS.get_font("body"),
                bg=self.theme["bg_card"], fg=self.theme["text_primary"]).pack(side=tk.LEFT)
        
        periods = sorted(period_topics.keys())
        self._period_var = tk.StringVar(value=str(periods[0]) if periods else "")
        combo = ttk.Combobox(control, textvariable=self._period_var,
                            values=[str(p) for p in periods], state="readonly", width=15)
        combo.pack(side=tk.LEFT, padx=4)
        
        self._period_table = DataTable(parent, theme=self.theme_name)
        self._period_table.pack(fill=tk.BOTH, expand=True, padx=8, pady=8)
        self._period_topics = period_topics
        
        def update(*args):
            try:
                p = int(self._period_var.get())
                if p in self._period_topics:
                    self._period_table.set_data(self._period_topics[p])
            except ValueError:
                pass
        
        combo.bind("<<ComboboxSelected>>", update)
        update()
    
    def _export_results(self):
        """Export results."""
        if self._result is None:
            messagebox.showwarning("No Results", "Run analysis first.")
            return
        
        filename = filedialog.asksaveasfilename(
            defaultextension=".xlsx", filetypes=[("Excel", "*.xlsx")],
            title="Export Results"
        )
        if not filename:
            return
        
        try:
            result = self._result
            is_dtm = self._method == "DTM"
            
            with pd.ExcelWriter(filename, engine="xlsxwriter") as w:
                if is_dtm:
                    result["topic_prevalence_evolution"].to_excel(w, sheet_name="Prevalence", index=False)
                    result["time_slice_info"].to_excel(w, sheet_name="Time Slices", index=False)
                    result["global_topics"].to_excel(w, sheet_name="Global Topics", index=False)
                    for t, df in result["topic_word_evolution"].items():
                        df.to_excel(w, sheet_name=t[:31], index=False)
                else:
                    result["topic_evolution"].to_excel(w, sheet_name="Evolution", index=False)
                    result["topic_prevalence"].to_excel(w, sheet_name="Prevalence", index=False)
                    result["period_stats"].to_excel(w, sheet_name="Period Stats", index=False)
                    for p, df in result["period_topics"].items():
                        df.to_excel(w, sheet_name=f"Period_{p}"[:31], index=False)
                
                # Add term evolution if computed
                if hasattr(self, '_term_evolution_df') and self._term_evolution_df is not None:
                    self._term_evolution_df.to_excel(w, sheet_name="Term Evolution", index=False)
            
            messagebox.showinfo("Success", f"Exported to:\n{filename}")
        except Exception as e:
            messagebox.showerror("Error", f"Export failed: {e}")
    
    def _create_term_evolution_tab(self, parent):
        """Create term evolution analysis tab."""
        # Controls
        control_frame = tk.Frame(parent, bg=self.theme["bg_card"])
        control_frame.pack(fill=tk.X, padx=8, pady=8)
        
        tk.Label(
            control_frame, text="Top N Terms:",
            font=FONTS.get_font("body"), bg=self.theme["bg_card"],
            fg=self.theme["text_primary"]
        ).pack(side=tk.LEFT, padx=(0, 4))
        
        self._term_top_n = tk.IntVar(value=15)
        top_n_spin = tk.Spinbox(
            control_frame, from_=5, to=30, textvariable=self._term_top_n, width=5
        )
        top_n_spin.pack(side=tk.LEFT, padx=4)
        
        tk.Label(
            control_frame, text="Plot Type:",
            font=FONTS.get_font("body"), bg=self.theme["bg_card"],
            fg=self.theme["text_primary"]
        ).pack(side=tk.LEFT, padx=(16, 4))
        
        self._term_plot_type = tk.StringVar(value="heatmap")
        plot_combo = ttk.Combobox(
            control_frame, textvariable=self._term_plot_type,
            values=["heatmap", "line", "area", "bump"], state="readonly", width=10
        )
        plot_combo.pack(side=tk.LEFT, padx=4)
        
        # Time span controls
        control_frame2 = tk.Frame(parent, bg=self.theme["bg_card"])
        control_frame2.pack(fill=tk.X, padx=8, pady=4)
        
        tk.Label(
            control_frame2, text="Time Span:",
            font=FONTS.get_font("body"), bg=self.theme["bg_card"],
            fg=self.theme["text_primary"]
        ).pack(side=tk.LEFT, padx=(0, 4))
        
        tk.Label(
            control_frame2, text="From:",
            font=FONTS.get_font("small"), bg=self.theme["bg_card"],
            fg=self.theme["text_muted"]
        ).pack(side=tk.LEFT, padx=(8, 2))
        
        self._term_year_from = tk.StringVar(value="")
        year_from_entry = tk.Entry(control_frame2, textvariable=self._term_year_from, width=6)
        year_from_entry.pack(side=tk.LEFT, padx=2)
        
        tk.Label(
            control_frame2, text="To:",
            font=FONTS.get_font("small"), bg=self.theme["bg_card"],
            fg=self.theme["text_muted"]
        ).pack(side=tk.LEFT, padx=(8, 2))
        
        self._term_year_to = tk.StringVar(value="")
        year_to_entry = tk.Entry(control_frame2, textvariable=self._term_year_to, width=6)
        year_to_entry.pack(side=tk.LEFT, padx=2)
        
        tk.Label(
            control_frame2, text="(leave empty for all years)",
            font=FONTS.get_font("small"), bg=self.theme["bg_card"],
            fg=self.theme["text_muted"]
        ).pack(side=tk.LEFT, padx=(8, 4))
        
        tk.Button(
            control_frame2, text="Compute & Plot",
            command=self._compute_term_evolution,
            bg=self.theme["bg_secondary"], fg=self.theme["text_primary"],
            font=FONTS.get_font("body"), relief=tk.FLAT, cursor="hand2"
        ).pack(side=tk.LEFT, padx=16)
        
        # Plot frame
        self._term_plot_frame = PlotFrame(parent, theme=self.theme_name, show_toolbar=True, show_ai_button=True)
        self._term_plot_frame.pack(fill=tk.BOTH, expand=True, pady=8)
        
        # Initial message
        fig, ax = self._term_plot_frame.get_figure()
        ax.text(0.5, 0.5, "Click 'Compute & Plot' to analyze term evolution",
               ha='center', va='center', fontsize=12)
        ax.axis('off')
        self._term_plot_frame.refresh()
    
    def _compute_term_evolution(self):
        """Compute and plot term evolution."""
        if not self.bib:
            messagebox.showwarning("No Data", "No data loaded.")
            return
        
        text_column = self.text_combo.get()
        time_column = self.time_combo.get()
        
        # Find valid columns
        if text_column not in self.bib.df.columns:
            for fb in ["Abstract", "Processed Title", "Title"]:
                if fb in self.bib.df.columns:
                    text_column = fb
                    break
        
        if time_column not in self.bib.df.columns:
            for fb in ["Year", "Publication Year", "PY"]:
                if fb in self.bib.df.columns:
                    time_column = fb
                    break
        
        try:
            from biblium import utilsbib
            
            top_n = self._term_top_n.get()
            
            # Get time span filter
            year_from_str = self._term_year_from.get().strip()
            year_to_str = self._term_year_to.get().strip()
            
            # Filter dataframe by time span if specified
            df_filtered_time = self.bib.df.copy()
            df_filtered_time[time_column] = pd.to_numeric(df_filtered_time[time_column], errors='coerce')
            
            if year_from_str:
                try:
                    year_from = int(year_from_str)
                    df_filtered_time = df_filtered_time[df_filtered_time[time_column] >= year_from]
                except ValueError:
                    pass
            
            if year_to_str:
                try:
                    year_to = int(year_to_str)
                    df_filtered_time = df_filtered_time[df_filtered_time[time_column] <= year_to]
                except ValueError:
                    pass
            
            if len(df_filtered_time) == 0:
                messagebox.showwarning("No Data", "No documents in the selected time span.")
                return
            
            # Compute term evolution
            term_evo_df = utilsbib.compute_term_evolution(
                df=df_filtered_time,
                text_column=text_column,
                time_column=time_column,
                top_n_terms=top_n,
                normalize=True,
            )
            
            self._term_evolution_df = term_evo_df
            
            if term_evo_df.empty:
                messagebox.showwarning("No Data", "No term evolution data computed.")
                return
            
            # Plot
            plot_type = self._term_plot_type.get()
            fig, ax = self._term_plot_frame.get_figure()
            ax.clear()
            fig.clear()
            
            # Get top terms
            term_avg = term_evo_df.groupby('Term')['Frequency'].mean()
            top_terms = term_avg.nlargest(top_n).index.tolist()
            df_filtered = term_evo_df[term_evo_df['Term'].isin(top_terms)]
            
            # Pivot for plotting
            pivot = df_filtered.pivot(index='Term', columns='Period', values='Frequency').fillna(0)
            pivot = pivot.reindex(top_terms)
            
            if plot_type == "heatmap":
                ax = fig.add_subplot(111)
                import seaborn as sns
                hm = sns.heatmap(
                    pivot, cmap='viridis',
                    annot=len(pivot.columns) <= 10,
                    fmt='.4f', ax=ax,
                    cbar_kws={'label': 'Relative Frequency', 'orientation': 'horizontal', 'pad': 0.15}
                )
                ax.set_xlabel("Year")
                ax.set_ylabel("Term")
                ax.set_title("Term Frequency Evolution (Relative Frequency per Year)")
                
            elif plot_type == "line":
                ax = fig.add_subplot(111)
                colors = plt.cm.get_cmap('tab20')(np.linspace(0, 1, len(top_terms)))
                
                for i, term in enumerate(top_terms):
                    if term in pivot.index:
                        ax.plot(pivot.columns, pivot.loc[term], label=term,
                               color=colors[i], linewidth=2, marker='o', markersize=4)
                
                ax.set_xlabel("Year")
                ax.set_ylabel("Relative Frequency")
                ax.set_title("Term Trends Over Time (Relative Frequency per Year)")
                ax.legend(bbox_to_anchor=(1.02, 1), loc='upper left', fontsize=8)
                ax.grid(False)
                ax.spines['top'].set_visible(False)
                ax.spines['right'].set_visible(False)
                
            elif plot_type == "area":
                ax = fig.add_subplot(111)
                colors = plt.cm.get_cmap('tab20')(np.linspace(0, 1, len(top_terms)))
                
                ax.stackplot(pivot.columns, pivot.values,
                            labels=pivot.index.tolist(), colors=colors, alpha=0.8)
                ax.set_xlabel("Year")
                ax.set_ylabel("Cumulative Relative Frequency")
                ax.set_title("Stacked Term Frequencies (Relative Frequency per Year)")
                ax.legend(bbox_to_anchor=(1.02, 1), loc='upper left', fontsize=8)
                ax.grid(False)
                
            elif plot_type == "bump":
                ax = fig.add_subplot(111)
                
                # Compute ranks
                df_filtered_copy = df_filtered.copy()
                df_filtered_copy['Rank'] = df_filtered_copy.groupby('Period')['Frequency'].rank(
                    ascending=False, method='min'
                )
                pivot_rank = df_filtered_copy.pivot(index='Period', columns='Term', values='Rank')
                
                periods = pivot_rank.index.tolist()
                colors = plt.cm.get_cmap('tab10')(np.linspace(0, 1, len(top_terms)))
                
                for i, term in enumerate(top_terms):
                    if term in pivot_rank.columns:
                        ranks = pivot_rank[term].values
                        ax.plot(periods, ranks, label=term, color=colors[i],
                               linewidth=3, marker='o', markersize=8)
                
                ax.invert_yaxis()
                ax.set_xlabel("Year")
                ax.set_ylabel("Rank")
                ax.set_title("Term Rank Evolution (Bump Chart)")
                ax.set_yticks(range(1, min(top_n + 1, 16)))
                ax.grid(False)
            
            fig.tight_layout()
            self._term_plot_frame.refresh()
            
        except Exception as e:
            import traceback
            messagebox.showerror("Error", f"Failed to compute: {e}\n\n{traceback.format_exc()}")
    
    def _create_info_content(self, parent):
        """Create Info tab with documentation."""
        info_text = """
DYNAMIC TOPIC MODELING
══════════════════════

Track how topics evolve over time periods.

AVAILABLE METHODS
─────────────────
• DTM (Dynamic Topic Model)
  - Topics evolve through time slices
  - Word distributions change over time
  - Captures vocabulary evolution
  - Best for tracking term changes
  
• STM (Structural Topic Model)
  - Incorporates document metadata
  - Topic prevalence varies with covariates
  - Better for hypothesis testing
  - Includes topic correlations

DTM APPROACH
────────────
• Divides corpus by time period
• Topics linked across periods
• Words can gain/lose relevance
• Shows topic evolution trajectory

STM APPROACH
────────────
• Metadata influences topic proportions
• Prevalence regression on time
• Content can vary with covariates
• More flexible modeling

PARAMETERS
──────────
• Number of topics
• Time period granularity
• Text preprocessing options
• Minimum documents per period

OUTPUT
──────
• Topic prevalence over time
• Topic word evolution
• Emerging/declining topics
• Topic correlation network
• Period-specific top words

USE CASES
─────────
• Track research front emergence
• Identify methodology shifts
• Understand field evolution
• Compare topic trajectories
"""
        text_widget = tk.Text(
            parent, font=FONTS.get_font("body"),
            bg=self.theme["bg_card"], fg=self.theme["text_primary"],
            wrap=tk.WORD, padx=16, pady=16, relief=tk.FLAT,
        )
        text_widget.insert("1.0", info_text)
        text_widget.config(state=tk.DISABLED)
        text_widget.pack(fill=tk.BOTH, expand=True)
        
        # Add right-click context menu for copy
        def show_context_menu(event):
            menu = tk.Menu(text_widget, tearoff=0)
            menu.add_command(label="Copy Selected", command=lambda: copy_selected(text_widget))
            menu.add_command(label="Copy All", command=lambda: copy_all(text_widget))
            menu.tk_popup(event.x_root, event.y_root)
        
        def copy_selected(widget):
            try:
                widget.config(state=tk.NORMAL)
                selected = widget.get(tk.SEL_FIRST, tk.SEL_LAST)
                widget.config(state=tk.DISABLED)
                widget.clipboard_clear()
                widget.clipboard_append(selected)
            except tk.TclError:
                pass  # No selection
        
        def copy_all(widget):
            widget.config(state=tk.NORMAL)
            content = widget.get("1.0", tk.END)
            widget.config(state=tk.DISABLED)
            widget.clipboard_clear()
            widget.clipboard_append(content.strip())
        
        text_widget.bind("<Button-3>", show_context_menu)  # Right-click
        text_widget.bind("<Control-c>", lambda e: copy_selected(text_widget))

    def set_bib(self, bib):
        """Set bibliometric data."""
        self.bib = bib
        self._result = None
        
        if bib and hasattr(bib, 'df'):
            available = [c for c in self.TEXT_COLUMNS if c in bib.df.columns]
            if available:
                self.text_combo.combobox['values'] = available
                self.text_combo.set(available[0])
            
            time_cols = [c for c in ["Year", "Publication Year", "PY"] if c in bib.df.columns]
            if time_cols:
                self.time_combo.combobox['values'] = time_cols
                self.time_combo.set(time_cols[0])



# -*- coding: utf-8 -*-
"""
Workspace Component
===================
Main content workspace with tabbed panels.
"""

import tkinter as tk
from tkinter import ttk
from typing import Callable, Dict, Optional, Type, Any

from biblium.gui.config import FONTS, LAYOUT, get_theme
from biblium.gui.core.events import event_bus, EventBus


class Workspace(tk.Frame):
    """
    Main workspace area with tabbed interface.
    
    Features:
    - Tabbed panel interface
    - Closable tabs
    - Panel registry
    - Lazy panel loading
    - Shared bib object across all panels
    """
    
    def __init__(
        self,
        parent,
        theme: str = "light",
        **kwargs
    ):
        self.theme_name = theme
        self.theme = get_theme(theme)
        
        super().__init__(parent, bg=self.theme["bg_secondary"], **kwargs)
        
        # Panel registry: panel_id -> panel_class
        self._panel_registry: Dict[str, Type] = {}
        
        # Active panels: tab_id -> panel_instance
        self._active_panels: Dict[str, tk.Frame] = {}
        
        # Tab mapping: tab_id -> panel_id
        self._tab_mapping: Dict[str, str] = {}
        
        # Shared bib object - THE KEY FIX
        self._shared_bib = None
        
        # Shared bib_group object for group analysis
        self._shared_bib_group = None
        
        # Shared state manager
        self._state = None
        
        self._create_widgets()
        self._create_welcome_tab()
        
        # Subscribe to events
        event_bus.subscribe(EventBus.PANEL_CHANGED, self._on_panel_changed)
        event_bus.subscribe(EventBus.DATASET_LOADED, self._on_dataset_loaded)
        event_bus.subscribe(EventBus.RESET_PANELS, self._on_reset_panels)
        event_bus.subscribe("group_created", self._on_group_created)
    
    def _create_widgets(self):
        """Create workspace widgets."""
        # Notebook for tabs
        self.notebook = ttk.Notebook(self)
        self.notebook.pack(fill=tk.BOTH, expand=True, padx=8, pady=8)
        
        # Configure notebook style
        style = ttk.Style()
        style.configure(
            "TNotebook",
            background=self.theme["bg_secondary"],
            borderwidth=0,
        )
        style.configure(
            "TNotebook.Tab",
            padding=[16, 8],
            font=FONTS.get_font("body"),
            background=self.theme["bg_secondary"],
            foreground=self.theme["text_muted"],
        )
        style.map(
            "TNotebook.Tab",
            background=[("selected", self.theme["bg_card"])],
            foreground=[("selected", self.theme["accent_primary"])],
        )
        
        # Bind tab change
        self.notebook.bind("<<NotebookTabChanged>>", self._on_tab_changed)
    
    def _create_welcome_tab(self):
        """Create the welcome/home tab."""
        welcome = tk.Frame(self.notebook, bg=self.theme["bg_secondary"])
        self.notebook.add(welcome, text="  🏠 Home  ")
        
        # Center content
        center = tk.Frame(welcome, bg=self.theme["bg_secondary"])
        center.place(relx=0.5, rely=0.4, anchor=tk.CENTER)
        
        tk.Label(
            center,
            text="📚",
            font=("Segoe UI", 64),
            bg=self.theme["bg_secondary"],
        ).pack()
        
        tk.Label(
            center,
            text="Biblium",
            font=("Segoe UI", 36, "bold"),
            bg=self.theme["bg_secondary"],
            fg=self.theme["text_primary"],
        ).pack(pady=(12, 4))
        
        tk.Label(
            center,
            text="Bibliometric Analysis Platform",
            font=FONTS.get_font("body"),
            bg=self.theme["bg_secondary"],
            fg=self.theme["text_muted"],
        ).pack()
        
        tk.Label(
            center,
            text="Load a dataset to get started",
            font=FONTS.get_font("small"),
            bg=self.theme["bg_secondary"],
            fg=self.theme["text_muted"],
        ).pack(pady=(24, 0))
        
        # Quick start button
        from biblium.gui.widgets.buttons import ThemedButton
        
        btn_frame = tk.Frame(center, bg=self.theme["bg_secondary"])
        btn_frame.pack(pady=24)
        
        ThemedButton(
            btn_frame,
            text="Load Dataset",
            icon="📂",
            style="primary",
            size="large",
            theme=self.theme_name,
            command=lambda: event_bus.emit(EventBus.PANEL_CHANGED, {"panel": "load"}),
        ).pack()
        
        self._active_panels["home"] = welcome
        self._tab_mapping["home"] = "home"
    
    @property
    def bib(self):
        """Get the shared bib object."""
        return self._shared_bib
    
    @bib.setter
    def bib(self, value):
        """Set the shared bib object and update all panels."""
        self._shared_bib = value
        self._update_all_panels_bib()
    
    def _update_all_panels_bib(self):
        """Update bib reference in all active panels."""
        for tab_id, panel in self._active_panels.items():
            if tab_id != "home" and hasattr(panel, 'bib'):
                panel.bib = self._shared_bib
    
    @property
    def bib_group(self):
        """Get the shared bib_group object."""
        return self._shared_bib_group
    
    @bib_group.setter
    def bib_group(self, value):
        """Set the shared bib_group object and update all panels."""
        self._shared_bib_group = value
        self._update_all_panels_bib_group()
    
    def _update_all_panels_bib_group(self):
        """Update bib_group reference in all active panels."""
        for tab_id, panel in self._active_panels.items():
            if tab_id != "home":
                # Set bib_group if panel has it
                if hasattr(panel, 'bib_group'):
                    panel.bib_group = self._shared_bib_group
                # Also try _bib_group for panels that use internal storage
                if hasattr(panel, '_bib_group'):
                    panel._bib_group = self._shared_bib_group
    
    def _on_group_created(self, data):
        """Handle group created event - update shared bib_group."""
        if isinstance(data, dict):
            bib_group = data.get("bib_group")
            if bib_group:
                self._shared_bib_group = bib_group
                self._update_all_panels_bib_group()
    
    def _on_dataset_loaded(self, data):
        """Handle dataset loaded event - update shared bib."""
        if isinstance(data, dict):
            bib = data.get("bib")
            if bib:
                self._shared_bib = bib
                self._update_all_panels_bib()
    
    def _on_reset_panels(self, data=None):
        """Handle reset panels event - close all tabs except Home and Load Dataset."""
        # Close all matplotlib figures first to prevent thread issues
        try:
            import matplotlib.pyplot as plt
            plt.close('all')
        except:
            pass
        
        # Get list of tabs to close (avoid modifying dict during iteration)
        tabs_to_close = []
        for tab_id, panel_id in self._tab_mapping.items():
            if panel_id not in ("home", "load"):
                tabs_to_close.append(tab_id)
        
        # Close each tab safely
        for tab_id in tabs_to_close:
            try:
                panel = self._active_panels.get(tab_id)
                if panel:
                    # Stop any active spinners or operations
                    if hasattr(panel, '_stop_active_spinners'):
                        panel._stop_active_spinners()
                    # Remove from notebook first
                    try:
                        self.notebook.forget(panel)
                    except:
                        pass
                    # Allow UI to update
                    self.update_idletasks()
                    # Destroy panel
                    try:
                        panel.destroy()
                    except:
                        pass
                # Clean up tracking
                if tab_id in self._active_panels:
                    del self._active_panels[tab_id]
                if tab_id in self._tab_mapping:
                    del self._tab_mapping[tab_id]
            except Exception as e:
                print(f"Error closing tab {tab_id}: {e}")
        
        # Clear shared bib
        self._shared_bib = None
    
    def register_panel(self, panel_id: str, panel_class: Type):
        """Register a panel class."""
        self._panel_registry[panel_id] = panel_class
    
    def open_panel(self, panel_id: str, **kwargs) -> Optional[tk.Frame]:
        """
        Open a panel (create tab if not exists).
        
        Parameters
        ----------
        panel_id : str
            Panel identifier.
        **kwargs
            Arguments to pass to panel constructor.
        
        Returns
        -------
        tk.Frame or None
            The panel instance.
        """
        # Check if already open
        for tab_id, pid in list(self._tab_mapping.items()):
            if pid == panel_id:
                # Select existing tab
                panel = self._active_panels.get(tab_id)
                if panel:
                    # Update bib reference in case it changed
                    if hasattr(panel, 'bib') and self._shared_bib:
                        panel.bib = self._shared_bib
                    # Update bib_group reference in case it changed
                    if self._shared_bib_group:
                        if hasattr(panel, '_bib_group'):
                            panel._bib_group = self._shared_bib_group
                        if hasattr(panel, 'bib_group'):
                            try:
                                panel.bib_group = self._shared_bib_group
                            except AttributeError:
                                pass  # Property might be read-only
                        # Refresh the panel if it has a refresh method
                        if hasattr(panel, 'refresh'):
                            try:
                                panel.refresh()
                            except Exception as e:
                                print(f"Error refreshing panel: {e}")
                    self.notebook.select(panel)
                    return panel
        
        # Check if panel is currently being created (prevent race condition)
        if not hasattr(self, '_creating_panels'):
            self._creating_panels = set()
        
        if panel_id in self._creating_panels:
            return None
        
        self._creating_panels.add(panel_id)
        
        try:
            # Get panel class
            panel_class = self._panel_registry.get(panel_id)
            if not panel_class:
                print(f"Warning: Panel '{panel_id}' not registered")
                return None
            
            # Always pass the shared bib to new panels
            if 'bib' not in kwargs and self._shared_bib:
                kwargs['bib'] = self._shared_bib
            
            # Always pass the shared bib_group to new panels
            if 'bib_group' not in kwargs and self._shared_bib_group:
                kwargs['bib_group'] = self._shared_bib_group
            
            # Pass state for settings panel
            if panel_id == "settings" and hasattr(self, '_state') and self._state:
                kwargs['state'] = self._state
            
            # Create panel
            try:
                panel = panel_class(self.notebook, theme=self.theme_name, **kwargs)
                
                # Also set bib_group after creation (in case panel creates it internally)
                if self._shared_bib_group and hasattr(panel, '_bib_group'):
                    panel._bib_group = self._shared_bib_group
                    
            except Exception as e:
                print(f"Error creating panel '{panel_id}': {e}")
                import traceback
                traceback.print_exc()
                return None
            
            # Get panel title
            title = getattr(panel, "title", panel_id.replace("_", " ").title())
            icon = getattr(panel, "icon", "")
            tab_text = f"  {icon} {title}  " if icon else f"  {title}  "
            
            # Add tab
            self.notebook.add(panel, text=tab_text)
            self.notebook.select(panel)
            
            # Track
            tab_id = str(id(panel))
            self._active_panels[tab_id] = panel
            self._tab_mapping[tab_id] = panel_id
            
            # Add close button behavior
            self._setup_tab_close(panel)
            
            return panel
        finally:
            # Always remove from creating set
            self._creating_panels.discard(panel_id)
    
    def _setup_tab_close(self, panel):
        """Setup middle-click and right-click to close tab."""
        # Middle click to close
        def close_tab_middle(event):
            try:
                tab_id = self.notebook.index("@%d,%d" % (event.x, event.y))
                if tab_id > 0:  # Don't close home tab
                    self.close_tab_by_index(tab_id)
            except tk.TclError:
                pass
        
        self.notebook.bind("<Button-2>", close_tab_middle)  # Middle click
        
        # Right-click context menu
        def show_tab_menu(event):
            try:
                tab_id = self.notebook.index("@%d,%d" % (event.x, event.y))
                if tab_id > 0:  # Don't show menu for home tab
                    self._show_tab_context_menu(event, tab_id)
            except tk.TclError:
                pass
        
        self.notebook.bind("<Button-3>", show_tab_menu)  # Right click
    
    def _show_tab_context_menu(self, event, tab_index: int):
        """Show context menu for tab."""
        menu = tk.Menu(self, tearoff=0)
        menu.add_command(label="Close Tab", command=lambda: self.close_tab_by_index(tab_index))
        menu.add_command(label="Close Other Tabs", command=lambda: self._close_other_tabs(tab_index))
        menu.add_command(label="Close All Tabs", command=self._close_all_tabs)
        menu.add_separator()
        menu.add_command(label="Refresh", command=lambda: self._refresh_tab(tab_index))
        
        try:
            menu.tk_popup(event.x_root, event.y_root)
        finally:
            menu.grab_release()
    
    def _close_other_tabs(self, keep_index: int):
        """Close all tabs except the specified one and home."""
        tabs_to_close = []
        for i in range(self.notebook.index("end") - 1, 0, -1):  # Reverse order, skip home (0)
            if i != keep_index:
                tabs_to_close.append(i)
        
        for idx in tabs_to_close:
            self.close_tab_by_index(idx)
    
    def _close_all_tabs(self):
        """Close all tabs except home."""
        for i in range(self.notebook.index("end") - 1, 0, -1):  # Reverse order, skip home (0)
            self.close_tab_by_index(i)
    
    def _refresh_tab(self, tab_index: int):
        """Refresh the specified tab."""
        try:
            panel = self.notebook.nametowidget(self.notebook.tabs()[tab_index])
            if hasattr(panel, 'refresh'):
                panel.refresh()
            elif hasattr(panel, '_on_dataset_loaded') and self._shared_bib:
                panel._on_dataset_loaded({"bib": self._shared_bib})
        except (tk.TclError, IndexError):
            pass
    
    def close_panel(self, panel_id: str):
        """Close a panel by ID."""
        for tab_id, pid in list(self._tab_mapping.items()):
            if pid == panel_id:
                panel = self._active_panels.get(tab_id)
                if panel:
                    self.notebook.forget(panel)
                    panel.destroy()
                    del self._active_panels[tab_id]
                    del self._tab_mapping[tab_id]
                break
    
    def close_tab_by_index(self, index: int):
        """Close tab by index."""
        if index <= 0:  # Don't close home tab
            return
        
        try:
            panel = self.notebook.nametowidget(self.notebook.tabs()[index])
            tab_id = str(id(panel))
            
            self.notebook.forget(index)
            panel.destroy()
            
            if tab_id in self._active_panels:
                del self._active_panels[tab_id]
            if tab_id in self._tab_mapping:
                del self._tab_mapping[tab_id]
        except (tk.TclError, IndexError):
            pass
    
    def get_active_panel(self) -> Optional[tk.Frame]:
        """Get currently active panel."""
        try:
            selected = self.notebook.select()
            current = self.notebook.nametowidget(selected)
            return current
        except tk.TclError:
            return None
    
    def _on_panel_changed(self, data):
        """Handle panel change event."""
        if isinstance(data, dict):
            panel_id = data.get("panel")
            kwargs = data.get("kwargs", {})
        else:
            panel_id = data
            kwargs = {}
        
        if panel_id:
            self.open_panel(panel_id, **kwargs)
    
    def _on_tab_changed(self, event):
        """Handle tab change event."""
        try:
            current = self.notebook.select()
            panel = self.notebook.nametowidget(current)
            tab_id = str(id(panel))
            panel_id = self._tab_mapping.get(tab_id, "home")
            
            event_bus.emit(EventBus.TAB_CHANGED, {"panel_id": panel_id, "tab_id": tab_id})
        except tk.TclError:
            pass
